#!/usr/bin/env python
# -*- coding: utf-8 -*-

import base64
import binascii
import logging

from django.conf import settings
try:
    from django.utils.module_loading import import_string
except ImportError:
    from django.utils.module_loading import import_by_path as import_string
from django.core.exceptions import ImproperlyConfigured

logger = logging.getLogger(__name__)


def get_provider():
    provider_class = getattr(settings, 'SHIBBOLETH_USER_PROVIDER', None)

    if provider_class is not None:
        provider_class = import_string(provider_class)
    else:
        raise ImproperlyConfigured('Missing SHIBBOLETH_USER_PROVIDER')

    return provider_class()


class HeadersProvider(object):
    def __init__(self):
        self.headers_map = getattr(settings, 'SHIBBOLETH_USER_HEADERS_MAP', {
            'first_name': 'HTTP_GIVENNAME',
            'last_name': 'HTTP_SN',
            'email': 'HTTP_MAIL',
        })
        self.fixed_values = {
            'is_active': getattr(settings, 'SHIBBOLETH_USER_DEFAULT_ACTIVE', True),
        }

    def __call__(self, request):
        logger.debug('Providing for %s', request)
        try:
            hmap = {
                key: self._get(request, key)
                for key in self.headers_map
            }
        except KeyError as e:
            logger.warning('Incomplete authentication missing: %s', e)
            return None

        hmap.update(self.fixed_values)
        return hmap

    def _get(self, request, key):
        return request.META[self.headers_map[key]]


class Base64HeadersProvider(HeadersProvider):
    def __init__(self):
        super(Base64HeadersProvider, self).__init__()
        self.values_to_decode = {'first_name', 'last_name'}

    def _get(self, request, key):
        value = super(Base64HeadersProvider, self)._get(request, key)
        if key in self.values_to_decode:
            try:
                value = base64.b64decode(value).decode('utf-8')
            except binascii.Error as e:
                raise ValueError('Cannot decode {}: decode({}) = {}'.format(
                    key, value, e))
        return value
